==> dnet-mapreduce-jobs-0.0.1-20131119.125713-139-sources.jar: eu/dnetlib/data/mapreduce/hbase/dedup/DedupBuildRootsMapper.java <==
package eu.dnetlib.data.mapreduce.hbase.dedup;

import java.io.IOException;
import java.util.Map;
import java.util.Set;

import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.io.ImmutableBytesWritable;
import org.apache.hadoop.hbase.mapreduce.TableMapper;
import org.apache.hadoop.hbase.util.Bytes;
import org.apache.hadoop.io.Text;

import com.google.common.base.Function;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.google.protobuf.InvalidProtocolBufferException;

import eu.dnetlib.data.mapreduce.hbase.dedup.config.DedupConfig;
import eu.dnetlib.data.mapreduce.hbase.dedup.config.DedupConfigLoader;
import eu.dnetlib.data.mapreduce.util.DedupRootUtils;
import eu.dnetlib.data.mapreduce.util.UpdateMerger;
import eu.dnetlib.data.proto.KindProtos.Kind;
import eu.dnetlib.data.proto.OafProtos.Oaf;
import eu.dnetlib.data.proto.RelTypeProtos.RelType;
import eu.dnetlib.data.proto.TypeProtos.Type;

public class DedupBuildRootsMapper extends TableMapper<Text, ImmutableBytesWritable> {

	private DedupConfig dedupConf;

	private final Set<String> entityNames = Sets.newHashSet(Iterables.transform(Lists.newArrayList(Type.values()), new Function<Type, String>() {

		@Override
		public String apply(final Type t) {
			return t.toString();
		}
	}));

	@Override
	protected void setup(final Context context) {
		dedupConf = DedupConfigLoader.load(context.getConfiguration().get("dedup.wf.conf"));
		System.out.println("dedup buildRoots mapper\nwf conf: " + dedupConf.toString());
	}

	@Override
	protected void map(final ImmutableBytesWritable rowkey, final Result value, final Context context) throws IOException, InterruptedException {
		// System.out.println("Find root mapping: " + new String(rowkey.copyBytes()));

		Map<byte[], byte[]> dedupRels = value.getFamilyMap(Bytes.toBytes(RelType.dedupRel.toString()));

		if ((dedupRels != null) && !dedupRels.isEmpty()) {

			if (!DedupRootUtils.isRoot(rowkey)) {

				final Text rootId = findRoot(dedupRels);
				// byte[] rootIdBytes = rootId.copyBytes();
				// byte[] rowkeyBytes = rowkey.copyBytes();

				context.getCounter(dedupConf.getEntityName(), "merged").increment(1);
				for (String family : dedupConf.getRootBuilderFamilies()) {

					// if (checkHack(rowkeyBytes, rootIdBytes, family)) {
					// context.getCounter("hack", "personResult skipped").increment(1);
					// } else {

					final Map<byte[], byte[]> map = value.getFamilyMap(Bytes.toBytes(family));
					if ((map != null) && !map.isEmpty()) {

						// if we're dealing with the entity CF
						if (entityNames.contains(family)) {
							final byte[] body = UpdateMerger.mergeBodyUpdates(map);
							context.write(rootId, new ImmutableBytesWritable(body));
						} else {
							for (byte[] o : map.values()) {

								if (!isRelMarkedDeleted(context, o)) {
									context.write(rootId, new ImmutableBytesWritable(o));
								} else {
									context.getCounter(family, "rel marked deleted").increment(1);
								}
							}
						}
					} // else {
						// System.err.println("empty family: " + family + "\nkey: " + sKey);
						// context.getCounter("DedupBuildRootsMapper", "empty family '" + family + "'").increment(1);
					// }
					// }
				}
			} else {
				context.getCounter(dedupConf.getEntityName(), "root").increment(1);
			}
		} else {
			context.getCounter(dedupConf.getEntityName(), "not in duplicate group").increment(1);
		}
	}

	protected boolean checkHack(final byte[] rowkey, final byte[] rootId, final String family) {
		return dedupConf.getEntityName().equals(Type.result.toString()) && 	// we're deduplicating the results
				family.equals(RelType.personResult.toString()) && 			// we're dealing with a personResult relation
				!rootId.equals(DedupRootUtils.newIdBytes(new String(rowkey), dedupConf.getDedupRun()));
	}

	private boolean isRelMarkedDeleted(final Context context, final byte[] o) {
		try {
			Oaf oaf = Oaf.parseFrom(o);
			return oaf.getKind().equals(Kind.relation) && oaf.getDataInfo().getDeletedbyinference();
		} catch (InvalidProtocolBufferException e) {
			context.getCounter("error", e.getClass().getName()).increment(1);
			return true;
		}
	}

	private Text findRoot(final Map<byte[], byte[]> dedupRels) {
		return new Text(Iterables.getOnlyElement(dedupRels.keySet()));
	}
}

==> dnet-mapreduce-jobs-0.0.2-20140417.161426-112-sources.jar: eu/dnetlib/data/mapreduce/hbase/dedup/DedupBuildRootsMapper.java <==
package eu.dnetlib.data.mapreduce.hbase.dedup;

import java.io.IOException;
import java.util.Map;
import java.util.Set;

import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.io.ImmutableBytesWritable;
import org.apache.hadoop.hbase.mapreduce.TableMapper;
import org.apache.hadoop.hbase.util.Bytes;
import org.apache.hadoop.io.Text;

import com.google.common.base.Function;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.google.protobuf.InvalidProtocolBufferException;

import eu.dnetlib.data.mapreduce.util.DedupRootUtils;
import eu.dnetlib.data.mapreduce.util.UpdateMerger;
import eu.dnetlib.data.proto.KindProtos.Kind;
import eu.dnetlib.data.proto.OafProtos.Oaf;
import eu.dnetlib.data.proto.RelTypeProtos.RelType;
import eu.dnetlib.data.proto.TypeProtos.Type;
import eu.dnetlib.pace.util.DedupConfig;
import eu.dnetlib.pace.util.DedupConfigLoader;

public class DedupBuildRootsMapper extends TableMapper<Text, ImmutableBytesWritable> {

	private DedupConfig dedupConf;

	private ImmutableBytesWritable ibw;

	private final Set<String> entityNames = Sets.newHashSet(Iterables.transform(Lists.newArrayList(Type.values()), new Function<Type, String>() {

		@Override
		public String apply(final Type t) {
			return t.toString();
		}
	}));

	@Override
	protected void setup(final Context context) {
		dedupConf = DedupConfigLoader.load(context.getConfiguration().get("dedup.wf.conf"));
		System.out.println("dedup buildRoots mapper\nwf conf: " + dedupConf.toString());

		ibw = new ImmutableBytesWritable();
	}

	@Override
	protected void map(final ImmutableBytesWritable rowkey, final Result value, final Context context) throws IOException, InterruptedException {
		// System.out.println("Find root mapping: " + new String(rowkey.copyBytes()));

		Map<byte[], byte[]> dedupRels = value.getFamilyMap(Bytes.toBytes(RelType.dedupRel.toString()));

		if ((dedupRels != null) && !dedupRels.isEmpty()) {

			if (!DedupRootUtils.isRoot(rowkey)) {

				final Text rootId = findRoot(dedupRels);
				// byte[] rootIdBytes = rootId.copyBytes();
				// byte[] rowkeyBytes = rowkey.copyBytes();

				context.getCounter(dedupConf.getEntityName(), "merged").increment(1);
				for (String family : dedupConf.getRootBuilderFamilies()) {

					// if (checkHack(rowkeyBytes, rootIdBytes, family)) {
					// context.getCounter("hack", "personResult skipped").increment(1);
					// } else {

					final Map<byte[], byte[]> map = value.getFamilyMap(Bytes.toBytes(family));
					if ((map != null) && !map.isEmpty()) {

						// if we're dealing with the entity CF
						if (entityNames.contains(family)) {
							final Oaf body = UpdateMerger.mergeBodyUpdates(context, map);

							emit(context, rootId, body.toByteArray());
						} else {
							for (byte[] o : map.values()) {

								if (!isRelMarkedDeleted(context, o)) {
									emit(context, rootId, o);
								} else {
									context.getCounter(family, "rel marked deleted").increment(1);
								}
							}
						}
					} // else {
						// System.err.println("empty family: " + family + "\nkey: " + sKey);
						// context.getCounter("DedupBuildRootsMapper", "empty family '" + family + "'").increment(1);
					// }
					// }
				}
			} else {
				context.getCounter(dedupConf.getEntityName(), "root").increment(1);
			}
		} else {
			context.getCounter(dedupConf.getEntityName(), "not in duplicate group").increment(1);
		}
	}

	private void emit(final Context context, final Text rootId, final byte[] value) throws IOException, InterruptedException {
		ibw.set(value);
		context.write(rootId, ibw);
	}

	private boolean checkHack(final byte[] rowkey, final byte[] rootId, final String family) {
		return dedupConf.getEntityName().equals(Type.result.toString()) && 	// we're deduplicating the results
				family.equals(RelType.personResult.toString()) && 			// we're dealing with a personResult relation
				!rootId.equals(DedupRootUtils.newIdBytes(new String(rowkey), dedupConf.getDedupRun()));
	}

	private boolean isRelMarkedDeleted(final Context context, final byte[] o) {
		try {
			Oaf oaf = Oaf.parseFrom(o);
			return oaf.getKind().equals(Kind.relation) && oaf.getDataInfo().getDeletedbyinference();
		} catch (InvalidProtocolBufferException e) {
			context.getCounter("error", e.getClass().getName()).increment(1);
			return true;
		}
	}

	private Text findRoot(final Map<byte[], byte[]> dedupRels) {
		return new Text(Iterables.getOnlyElement(dedupRels.keySet()));
	}
}
